%code requires {

  #define YYERROR_VERBOSE
  #include <iostream>
  #include "instructions.h"
  class WhileLexer; // forward declaration

} // Yacc Deklarationen

%defines
%define parser_class_name {WhileParser}
%parse-param { WhileLexer &lexer }
%parse-param { WhileProgram** result }
%lex-param { WhileLexer &lexer }

%union
{
    std::string* stringval;
    int* intval;
    WhileInstruction* instrval;
    WhileProgram* progval;
};

%skeleton "lalr1.cc"
%start S

%token SEMICOLON NEQ WHILE DO DONE ASSIGN ADD SUB
%token <intval> NUM
%token <stringval> ID

%left NEQ
%left ADD SUB

%code {
    static int yylex(yy::WhileParser::semantic_type* yylval, WhileLexer &scanner);
}

%% // Produktionen
S
    : program
    {
        *result = $<progval>1;
    }
    ;

program
    :
    {
        $<progval>$ = new WhileProgram();
    }

    | program instruction
    {
        $<progval>$ = $<progval>1;
        $<progval>$->Instructions.push_back($<instrval>2);
    }
    ;

instruction
    : ID ASSIGN ID ADD NUM SEMICOLON
    {
        $<instrval>$ = new Assignment( *$<stringval>1, *$<stringval>3, *$<intval>5 );
        delete $<stringval>1;
        delete $<intval>5;
        delete $<stringval>3;
    }

    | ID ASSIGN ID SUB NUM SEMICOLON
    {
        $<instrval>$ = new Assignment( *$<stringval>1, *$<stringval>3, (-1) * (*$<intval>5) );
        delete $<stringval>1;
        delete $<intval>5;
        delete $<stringval>3;
    }

    | WHILE ID NEQ NUM DO program DONE
    {
        $<instrval>$ = new WhileLoop( *$<stringval>2, *$<intval>4, $<progval>6 );
        delete $<stringval>2;
        delete $<intval>4;
    }
    ;

%%

#include "whilelexer.h"

void yy::WhileParser::error(const std::string& s)
{
    std::cerr << "SYNTACTIC ERROR (Line " << lexer.GetCurrentLineNumber() << "): " << s << std::endl;
}

static int yylex(yy::WhileParser::semantic_type* yylval, WhileLexer &lexer)
{
    return lexer.yylex(yylval);
}
